/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#define OWL_TRACE OWL_TRACE_INFO
#define OWL_FILE "owl_wdt"

#include <owl/core/owl_defs.h>
#include <owl/core/owl_debug.h>
#include <owl/drivers/owl_wdt.h>


void
owlwdt_set_period(uint32_t ms)
{
        owl_info("ms:%d", ms);

        /* Enable write access to IWDG_PR and IWDG_RLR registers */
        IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);

        /* IWDG counter clock: LSI/32 */
        IWDG_SetPrescaler(IWDG_Prescaler_128);

        /* Set counter reload value to obtain <ms> IWDG TimeOut.
           Reload value = (ms / 1000) * (LsiFreq / 128) */
        IWDG_SetReload(ms * (STM32_BOARD_LSI_FREQ / 1000) / 128);

        IWDG_ReloadCounter();
}


void
owlwdt_enable(void)
{
        owl_info("enter");

        /* Make sure that the LSI clock is enabled in owl_board.c since the
           watchdog runs on internal 40 khz clock */
        
        /* The watchdog counter clock is stopped when the core is halted */
        DBGMCU_Config(DBGMCU_IWDG_STOP, ENABLE);
        
        IWDG_Enable();
        IWDG_ReloadCounter();
}

void
owlwdt_disable(void)
{
        owl_info("enter");
        owl_err("not implemented");
}

void
owlwdt_clear(void)
{
        owl_vdbg("enter");
        IWDG_ReloadCounter();
}

int
owlwdt_trigged(void)
{
        /* Check if the system has resumed from IWDG reset */
        if (RCC_GetFlagStatus(RCC_FLAG_IWDGRST) != RESET) {
                RCC_ClearFlag();
                return 1;
        }
        
        return 0;
}

