/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef OWL_AP_H
#define OWL_AP_H

#include <wl_api.h>

typedef void (owlap_link_cb_t)(void *ctx, int connected);
typedef int (owlap_rand_cb_t)(void);

int owlap_init(wl_fw_read_cb_t *fw_read_cb, void *fw_read_ctx,
               owlap_link_cb_t *link_cb, void *link_ctx);

/* Start the AP mode.
 *
 * @param ssid The SSID that the AP will use.
 * @param passphrase The WPA/WPA2 PSK passphrase as a NUL-terminated string.
 * @param chan Channel to use (1-14).
 * @param enc_type Type of encryption to use. 
 * @param auth_mode Authentication mode to use. 
 * @param rand_cb Function to get a random value.
 *                Can be NULL. In this case the owltmr_get_ms() will
 *                be used instead.
 * @return 
 *  - 0 on success
 *  - OWL_ERR_PARAM
 *  - OWL_ERR_NOTSUPP
 *  - OWL_ERR_STATE
 *  - OWL_FAILURE
 */
int owlap_start(const struct wl_ssid_t *ssid, 
                const char *passphrase, 
                int chan,
                enum wl_enc_type enc_type,
                enum wl_auth_mode auth_mode,
                owlap_rand_cb_t rand_cb);
int owlap_stop(void);
int owlap_running(void);

extern const struct owlsh_cmd cmd_ap;

#endif



