/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#define OWL_TRACE OWL_TRACE_ERR

#include <wl_api.h>
#include <owl/core/owl_loop.h>
#include <owl/core/owl_debug.h>

static void rx_isr(void *ctx);
static void print_progress(void);
static void read_data(void);

static volatile int rx_pending;
static uint8_t rx_buf[WL_MAX_PKT_LEN];
static unsigned int pkt;

wl_err_t
owlloop_run(wl_fw_read_cb_t *fw_read_cb, void *fw_read_ctx,
            int start_size, int step_size, int cnt)
{
        enum wl_host_attention_mode mode;
        wl_err_t err;
        int tx_len = start_size;
        int tx_cnt = cnt;

        static uint8_t tx_buf[WL_MAX_PKT_LEN] = {
                0x0, 0x0, 0x3, 0x0, 0x6, 0x0, 0x0, 0x0
        };

        static uint8_t align_req[] = {
                0x1e, 0x00, /* size */
                0x07, 0x04, /* type, id */
                0x06, 0x0e, /* hdr, pad added:18 */
                
                0x01, 0x00, 0x00, 0x00, /* transid */
                0x20, 0x10, /* min_size, pad_size */
		0x01, /* attention: poll/sdio */
		0x06, /* ul hdr size */
                0x00, /* swap */
		0x00, 0x00, 0x00, /* reserved [3] */
                
                0x00, 0x00, 0x00, 0x00, 0x00, 0x00,  /* pad */
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 /* pad */
        };

        pkt = 0;

        owl_dbg("fw download");

        if ((err = wl_transport_init(fw_read_cb, fw_read_ctx,
                                     &mode)) != WL_SUCCESS) {
                owl_err("transport init failed, err:%d", err);
                return err;
        }
        wl_register_rx_isr(rx_isr, NULL);

        rx_pending = 0;    
        align_req[12] = mode;
        owl_info("tx alignment req");
        wl_tx(align_req, sizeof(align_req));

        while (!rx_pending)
                wl_poll();

        read_data();

        owl_os_printf("%d loopback msgs start\n\r", cnt);

        while (tx_cnt--) {
                int i;

                tx_buf[0] = (tx_len - 2) % 256; /* le */
                tx_buf[1] = (tx_len - 2) / 256; /* le */
                for (i = 8; i < tx_len; i++)
                        tx_buf[i] = i % 256;

                rx_pending = 0;

                wl_tx(tx_buf, tx_len);

                while (!rx_pending)
                        wl_poll();

                read_data();

                tx_len += step_size;
                if (tx_len > WL_MAX_PKT_LEN)
                        tx_len = start_size;
        }
    
        owl_os_printf("\n\r\n\r");
        owl_os_printf("%d loopback msgs ok\n\r", cnt);
        return WL_SUCCESS;
}

static void
rx_isr(void *ctx)
{
        (void) ctx;
        rx_pending = 1;
}

    
static void
read_data(void)
{
        uint16_t i;
        uint16_t rx_len = sizeof(rx_buf);

        wl_rx(rx_buf, &rx_len);

        if (rx_len == 0) {
                owl_os_printf("rx_len:0\n\r");
                return;
        }
        
        if (pkt == 0) {
                owl_os_printf("rx alignment cfm, len:%d\n\r", rx_len);
                pkt++;
                return;
        }

        /* Remove padding and perform test */
        rx_len -= rx_buf[5];
        for (i = 8; i < rx_len; ++i) {
                if (rx_buf[i] != i % 256) {
                        owl_os_printf("fail on pkt:%u offset:%u "
                                   "got:0x%02x expected:0x%02x\n\r",
                                   pkt, i, rx_buf[i], i % 256);
                        break;
                }
        }

        print_progress();
        pkt++;
}

static void
print_progress(void)
{
        static int progress = 0;
        if (progress && progress % (32 * 64) == 0)
                owl_os_printf("\n\r");
        if (progress % 64 == 0)
                owl_os_printf(".");
        progress++;
}

