/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef OWL_NET_H
#define OWL_NET_H

#include <lwip/netif.h>
#include <lwip/ip_addr.h>


/**
 * Initialize the network stack and start the network device driver. Should be
 * called prior to any other function.
 *
 */
struct netif *owlnet_init(void (*ipstatus_cb)(struct netif *, void *ctx),
                          void *ctx);

/**
 * Disable the network interface. Network connections such as TCP sockets
 * will be kept since TCP/IP timers will still be running.
 *
 * @note: This function is optional and is only for advanced usage.
 *
 */
void owlnet_shutdown(void);

/**
 * Start TCP/IP timers explicitly. TCP/IP timers will always start upon
 * owlnet_linkup().
 *
 * @note: This function is optional and is only for advanced usage.
 *
 */
void owlnet_start(void);

/**
 * Stop TCP/IP timers.
 *
 * @note: This function is optional and is only for advanced usage.
 *
 */
void owlnet_stop(void);

/**
 * Should be called when the network interface has link, e.g. cable inserted
 * or connected to Wi-Fi access point.
 *
 */
void owlnet_linkup(void);

/** 
 * Should be called when the network interface link is down, e.g. lost
 * connection to access point.
 *
 */
void owlnet_linkdown(void);

/**
 * Enable/disable dhcp.
 *
 */
void owlnet_set_dhcp(int enable);

/**
 * Returns the current dhcp configuration.
 *
 */
int owlnet_get_dhcp(void);

/**
 * Set static IP address. Automatically disables DHCP.
 *
 */
void owlnet_set_addr(const struct ip_addr *ip,
                     const struct ip_addr *netmask,
                     const struct ip_addr *gw,
                     const struct ip_addr *dns);

int owlnet_is_hostname(const char *str);


/**
 * Access the lwip netif struct.
 *
 */
struct netif *owlnet_netif(void);

#ifndef ip_ntoa /* defined by lwip-1.3.2+ but not by 1.3.0 */
# define ip_ntoa(addr) \
        ((addr != NULL) ? inet_ntoa(*((struct in_addr*)(addr))) : "NULL")
#endif
#define ip_aton ipaddr_aton
#if 0 /* need for < lwip-1.4.0 */
int ip_aton(const char *cp, struct ip_addr *addr);
#endif

int lwip_error(err_t err);

extern const struct owlsh_cmd cmd_ifconfig;
extern const struct owlsh_cmd cmd_dhcp;


#endif
