/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef OWL_WIFI_H
#define OWL_WIFI_H

#include <wl_api.h>

typedef void (owlwifi_link_cb_t)(void *ctx, int connected);

int owlwifi_init(wl_fw_read_cb_t *fw_read_cb, void *fw_read_ctx,
                 owlwifi_link_cb_t *link_cb, void *link_ctx);
int owlwifi_set_conf(const struct wl_ssid_t *ssid,
                     const struct wl_mac_addr_t *bssid,
                     const char *key, int auth);
int owlwifi_start(void);
int owlwifi_stop(void);
int owlwifi_running(void);
const char *owlwifi_ssid_str(const struct wl_ssid_t *ssid);

enum owlwifi_connect_state {
        OWLWIFI_DISCONNECTED,
        OWLWIFI_CONNECTED,
        OWLWIFI_CONNECTING,
};

enum owlwifi_auth_mode {
        OWLWIFI_AUTH_AUTO = 0,
        OWLWIFI_WEP,
        OWLWIFI_WPA_PSK,
        OWLWIFI_WPA2_PSK
};

enum owlwifi_connect_state owlwifi_conn_state(void);

extern const struct owlsh_cmd cmd_scan;
extern const struct owlsh_cmd cmd_connect;
extern const struct owlsh_cmd cmd_wpass;
extern const struct owlsh_cmd cmd_dpass;
extern const struct owlsh_cmd cmd_powersave;
extern const struct owlsh_cmd cmd_psconf;
extern const struct owlsh_cmd cmd_setkey;

#endif
